/*
 * Package : Coap
 * Author : S. Hamblett <steve.hamblett@linux.com>
 * Date   : 12/04/2017
 * Copyright :  S.Hamblett
 */

library coap;

import 'dart:async';
import 'dart:collection';
import 'dart:convert' as convertor;
import 'dart:io';
import 'dart:math';
import 'dart:typed_data';
import 'package:typed_data/typed_data.dart' as typed;
import 'package:logging/logging.dart' as logging;
import 'package:collection/collection.dart' as collection;
import 'package:hex/hex.dart' as hex;
import 'package:executor/executor.dart' as tasking;
import 'package:event_bus/event_bus.dart' as events;
import 'package:string_scanner/string_scanner.dart' as scanner;
import 'package:date_format/date_format.dart' as df;

/// The Coap package exported interface

part 'src/coap_client.dart';

part 'src/coap_option_type.dart';

part 'src/coap_option.dart';

part 'src/coap_block_option.dart';

part 'src/coap_code.dart';

part 'src/coap_message.dart';

part 'src/coap_empty_message.dart';

part 'src/coap_request.dart';

part 'src/coap_link_format.dart';

part 'src/coap_link_attribute.dart';

part 'src/coap_web_link.dart';

part 'src/coap_response.dart';

part 'src/coap_message_type.dart';

part 'src/coap_config.dart';

part 'src/coap_observe_client_relation.dart';

part 'src/coap_media_type.dart';

part 'src/coap_constants.dart';

part 'src/log/coap_ilogger.dart';

part 'src/log/coap_null_logger.dart';

part 'src/log/coap_console_logger.dart';

part 'src/log/coap_log_manager.dart';

part 'src/deduplication/coap_ideduplicator.dart';

part 'src/deduplication/coap_noop_deduplicator.dart';

part 'src/deduplication/coap_crop_rotation_deduplicator.dart';

part 'src/deduplication/coap_sweep_deduplicator.dart';

part 'src/deduplication/coap_deduplicator_factory.dart';

part 'src/util/coap_util.dart';

part 'src/util/coap_byte_array_util.dart';

part 'src/util/coap_scanner.dart';

part 'src/specification/coap_ispec.dart';

part 'src/specification/drafts/coap_draft03.dart';

part 'src/specification/drafts/coap_draft08.dart';

part 'src/specification/drafts/coap_draft12.dart';

part 'src/specification/drafts/coap_draft13.dart';

part 'src/specification/drafts/coap_draft18.dart';

part 'src/codec/coap_imessage_encoder.dart';

part 'src/codec/encoders/coap_message_encoder.dart';

part 'src/codec/encoders/coap_message_encoder03.dart';

part 'src/codec/encoders/coap_message_encoder08.dart';

part 'src/codec/encoders/coap_message_encoder12.dart';

part 'src/codec/encoders/coap_message_encoder13.dart';

part 'src/codec/encoders/coap_message_encoder18.dart';

part 'src/codec/decoders/coap_message_decoder03.dart';

part 'src/codec/decoders/coap_message_decoder08.dart';

part 'src/codec/decoders/coap_message_decoder12.dart';

part 'src/codec/decoders/coap_message_decoder13.dart';

part 'src/codec/decoders/coap_message_decoder18.dart';

part 'src/codec/coap_imessage_decoder.dart';

part 'src/codec/decoders/coap_message_decoder.dart';

part 'src/codec/datagram/coap_datagram_reader.dart';

part 'src/codec/datagram/coap_datagram_writer.dart';

part 'src/net/coap_iendpoint.dart';

part 'src/net/coap_endpoint.dart';

part 'src/net/coap_endpoint_manager.dart';

part 'src/net/coap_imatcher.dart';

part 'src/net/coap_matcher.dart';

part 'src/net/coap_imessage_deliverer.dart';

part 'src/net/coap_client_message_deliverer.dart';

part 'src/net/coap_ioutbox.dart';

part 'src/net/coap_internet_address.dart';

part 'src/net/coap_exchange.dart';

part 'src/stack/coap_blockwise_status.dart';

part 'src/stack/coap_ilayer.dart';

part 'src/stack/coap_abstract_layer.dart';

part 'src/stack/coap_blockwise_layer.dart';

part 'src/stack/coap_ientry.dart';

part 'src/stack/coap_chain.dart';

part 'src/stack/coap_layer_stack.dart';

part 'src/stack/coap_observe_layer.dart';

part 'src/stack/coap_reliability_layer.dart';

part 'src/stack/coap_token_layer.dart';

part 'src/stack/coap_stack.dart';

part 'src/observe/coap_observe_relation.dart';

part 'src/observe/coap_observing_endpoint.dart';

part 'src/observe/coap_observe_notification_orderer.dart';

part 'src/resources/coap_iresource.dart';

part 'src/resources/coap_resource_attributes.dart';

part 'src/threading/coap_iexecutor.dart';

part 'src/threading/coap_executor.dart';

part 'src/endpoint/resources/coap_endpoint_resource.dart';

part 'src/endpoint/resources/coap_remote_resource.dart';

part 'src/channel/coap_ichannel.dart';

part 'src/channel/coap_udp_channel.dart';

part 'src/channel/coap_inetwork.dart';

part 'src/channel/coap_network_udp.dart';

part 'src/channel/coap_network_manager.dart';

part 'src/event/coap_event_bus.dart';

part 'src/coap_defined_address.dart';
